import processing.core.*; 
import processing.xml.*; 

import processing.opengl.*; 
import controlP5.*; 

import java.applet.*; 
import java.awt.Dimension; 
import java.awt.Frame; 
import java.awt.event.MouseEvent; 
import java.awt.event.KeyEvent; 
import java.awt.event.FocusEvent; 
import java.awt.Image; 
import java.io.*; 
import java.net.*; 
import java.text.*; 
import java.util.*; 
import java.util.zip.*; 
import java.util.regex.*; 

public class _processing__eid300_room_scan extends PApplet {

/*  Plots a 3D mesh from a text file containing a set 
    of points defined in a spherical coordinate system; 
    coordinates are comma-delimited, and points are
    separated by a newline character. 
    
    The total number of lines in the file should be a
    square number.
*/




public void setup() {
  size(800, 600, OPENGL);
  
  // Load texture and data from default files
  img = loadImage("default.png");
  mesh = getData("default.txt");
  meshBool = determineFaces(distFromCenter);

  /* NORMALIZED allows texture coordinates to
       be defined relative to 1 in drawMesh()*/
  textureMode(NORMALIZED);
  
  enableScrollWheel();
  menuSetup();
}

public void draw() {
  // Update colors based on color selections
  bgColor = color1.getColorValue();
  lineColor = color2.getColorValue();
  fillColor = color3.getColorValue();
  
  background(bgColor);
  
  // Plot points
  pushMatrix();
  lights();
  translate(width/2-yzTrans.arrayValue()[0],
            height/2-yzTrans.arrayValue()[1],depthTrans);
  rotateCamera();
  drawProperties();
  drawMesh(mesh);
  popMatrix();
  
  // Draw menu
  hint(DISABLE_DEPTH_TEST);
  strokeWeight(1);
  controlP5.draw();
  hint(ENABLE_DEPTH_TEST);
}
public void enableScrollWheel() {
  /* Allows the user to zoom in/out using scroll wheel 
      http://wiki.processing.org/index.php/Wheel_mouse
      Code by Rick Companje */
  addMouseWheelListener(
    new java.awt.event.MouseWheelListener() { 
      public void mouseWheelMoved(java.awt.event.MouseWheelEvent evt) { 
        zoom(evt.getWheelRotation());
      }
    }
  ); 
}

public void keyPressed() {
  // Allows zooming and changing depth via the arrow keys
  if (key == CODED) {
    switch(keyCode) {
      case UP:
        zoom(1);
        break;
      case DOWN:
        zoom(-1);
        break;
      case RIGHT:
        if (depthTrans < DEPTH_TRANS_MAX)
          depthTrans += depthTransRate;
          controlP5.controller("Depth").setValue(depthTrans);
        break;
      case LEFT:
        if (depthTrans > 0)
          depthTrans -= depthTransRate;      
          controlP5.controller("Depth").setValue(depthTrans);
        break;
    }
  }
}

public void rotateCamera() {  
  // Rotate the camera on a mouse click+drag
  if (mousePressed) {
    /* Check to see if menus are open 
        Don't rotate if the mouse is above a menu */
    if (!( menu.isOpen() && mouseX < MENU_XBOUND ) &&
        !( colors.isOpen() && mouseX > width-MENU_XBOUND 
            && mouseY < COLORS_MENU_HEIGHT ) ) {
      currentRotation[0] += RCONST*(pmouseY-mouseY);
      currentRotation[1] += RCONST*(pmouseX-mouseX);
  
      /* Rotation about the x-axis is 
          limited between 0 and PI */
      if (currentRotation[0] > PI)
        currentRotation[0] = PI;
      if (currentRotation[0] < 0)
        currentRotation[0] = 0;

      currentRotation[1] %= TWO_PI;
    }
  }
  rotateX(currentRotation[0]);
  rotateZ(currentRotation[1]);
}

public void zoom(int delta) {
  /* Positive delta means zoom in 
     Negative delta means zoom out */
     
  zoomScale += delta*ZOOM_RATE;
  if (zoomScale < ZOOM_RANGE[0])
    zoomScale = ZOOM_RANGE[0];
  if (zoomScale > ZOOM_RANGE[1])
    zoomScale = ZOOM_RANGE[1];
  controlP5.controller("Zoom").setValue(zoomScale);
}
// Controls the rate of rotation on a mouse click+drag
final float RCONST = .01f; // Weighting of rotation amount based on mouse movements
final float ZOOM_RATE = .3f;   // Scroll wheel zooming rate
final float ZOOM_RANGE[] = {.2f,50};  // Max and min zooms
final int DEPTH_TRANS_MAX = 410; // How far in the camera can go

// Constants for defining axes and lines
final int X_AXIS_COLOR = 0xffFF0000;
final int Y_AXIS_COLOR = 0xff00FF00;
final int Z_AXIS_COLOR = 0xff0000FF;
final int AXIS_LENGTH = 25;
final int AXIS_THICKNESS = 4;
final int LINE_THICKNESS = 2;

// Menu-related constants
final int MENU_XBOUND = 100;
final int MENU_BG_COLOR = color(0,230);
final int BUTTON_WIDTH = 25;
final int COLORS_MENU_HEIGHT = 230;

// Different plotting shapes to choose from
final int[] SHAPES = {POINTS, LINES, TRIANGLES, 
                      TRIANGLE_STRIP, QUADS, QUAD_STRIP};

final int MAX_SMOOTHING = 20; // Max number of times the mesh can be smoothed

/* Weights for smoothing; points are multiplied 
    by the reciprocal of these values */
final int MAIN_WEIGHT = 2;   // For the current point being smoothed 
final int SIDE_WEIGHT = 12; // For points directly adjacent to the current point
final int SIDE_WEIGHT2 = 9; // For points on the boundaries of the mesh
final int DIAG_WEIGHT = 24; // For points diagonal to the current point
public void controlEvent(ControlEvent theEvent) {
  // Event handling for menu clicks
  switch(theEvent.controller().id()) { // Check ID
    case(1): // maximumR adjust
      maximumR = theEvent.controller().value();
      meshBool = determineFaces(distFromCenter);
      break;
    case(2): // Depth translation
      depthTrans = (int) theEvent.controller().value();
      break;
    case(3): // Zoom in/out
      zoomScale = theEvent.controller().value();
      break;
    case(4): // Smoothing
      smoothingIterations = (int) theEvent.controller().value();
      smoothMultiple(smoothingIterations);
      break;
    case(5): // Shape
      whichShape = SHAPES[(int) theEvent.controller().value()-1];
      break;
    case(6): // Load texture
      imgLocation = selectInput();
      if (imgLocation != null)
        img = loadImage(selectInput());
      break;
    case(7): // getData()
      mesh = getData(selectInput());
      meshBool = determineFaces(distFromCenter);
      break;
  }
}

public void menuSetup() {
  // menuSetup() configures the main menu and colors menu
  controlP5 = new ControlP5(this);
  controlP5.setAutoDraw(false); 
  
  menu = controlP5.addGroup("Menu",0,10);
  menu.setBackgroundHeight(height);
  menu.setBackgroundColor(MENU_BG_COLOR);
  menu.setWidth(MENU_XBOUND);
  
  // Toggle settings with clickable buttons
  controlP5.addToggle("wireframe",wireframe,25,10,BUTTON_WIDTH,BUTTON_WIDTH).setGroup(menu);
  controlP5.addToggle("useTexture",useTexture,25,60,BUTTON_WIDTH,BUTTON_WIDTH).setGroup(menu);
  controlP5.addToggle("showAxes",showAxes,25,110,BUTTON_WIDTH,BUTTON_WIDTH).setGroup(menu);
  controlP5.addToggle("showLines",showLines,25,160,BUTTON_WIDTH,BUTTON_WIDTH).setGroup(menu);
  
  // Rename button labels
  controlP5.controller("useTexture").setLabel("Use Texture");
  controlP5.controller("showAxes").setLabel("Show Axes");
  controlP5.controller("showLines").setLabel("Show Lines");
  
  // Add sliders for camera and rendering settings
  controlP5.addSlider("Threshold",0,300,300,0,210,MENU_XBOUND,BUTTON_WIDTH).setId(1);
  controlP5.controller("Threshold").setGroup(menu);
  controlP5.addSlider("Depth",0,DEPTH_TRANS_MAX,0,0,240,MENU_XBOUND,BUTTON_WIDTH).setId(2);
  controlP5.controller("Depth").setGroup(menu);
  controlP5.addSlider("Zoom",ZOOM_RANGE[0],ZOOM_RANGE[1],0,0,270,MENU_XBOUND,BUTTON_WIDTH).setId(3);
  controlP5.controller("Zoom").setGroup(menu);
 
  controlP5.addSlider("Smoothing",0,MAX_SMOOTHING,2,0,300,MENU_XBOUND,BUTTON_WIDTH/2).setId(4);
  Slider smoothSlider = (Slider) controlP5.controller("Smoothing");
  smoothSlider.setGroup(menu);
  smoothSlider.setSliderMode(Slider.FLEXIBLE);
  smoothSlider.setNumberOfTickMarks(MAX_SMOOTHING+1);

  controlP5.addSlider("Shape",1,SHAPES.length,SHAPES.length,0,330,MENU_XBOUND,BUTTON_WIDTH/2).setId(5);
  Slider shapeSlider = (Slider) controlP5.controller("Shape");
  shapeSlider.setGroup(menu);
  shapeSlider.setSliderMode(Slider.FLEXIBLE);
  shapeSlider.setNumberOfTickMarks(SHAPES.length);
  
  // Buttons to load texture/data from file
  controlP5.addButton("Load texture",0,0,380,MENU_XBOUND,20).setId(6);
  controlP5.addButton("Load data",0,0,400,MENU_XBOUND,20).setId(7);
  controlP5.controller("Load texture").setGroup(menu);
  controlP5.controller("Load data").setGroup(menu);
  
  /* addSlider2D(name, xMin, xMax, yMin, yMax, defaultX, defaultY, 
                 xPos, yPos, width, height); */
  yzTrans = controlP5.addSlider2D("YZ-Translate",-width/4,width/4,
                  -height/4,height/4,width/4,height/4,0,height-125,MENU_XBOUND,MENU_XBOUND);
  yzTrans.setGroup(menu);
  
  // Colors menu group
  colors = controlP5.addGroup("Colors",width-100,10);
  colors.setBackgroundHeight(COLORS_MENU_HEIGHT);
  colors.setBackgroundColor(MENU_BG_COLOR);
  colors.setWidth(MENU_XBOUND);
  
  // Add color pickers to colors menu
  color1 = controlP5.addColorPicker("BG",0,175,MENU_XBOUND,10);
  color2 = controlP5.addColorPicker("Lines",0,15,MENU_XBOUND,10);
  color3 = controlP5.addColorPicker("Fill",0,95,MENU_XBOUND,10);
  color1.setColorValue(bgColor);
  color2.setColorValue(lineColor);
  color3.setColorValue(fillColor);
  color1.setGroup(colors);
  color2.setGroup(colors);
  color3.setGroup(colors);
  
  // Labels for color pickers
  controlP5.addTextlabel("BGLabel","Background",5,165);
  controlP5.addTextlabel("LinesLabel","Lines",5,5);
  controlP5.addTextlabel("FillLabel","Fill",5,85);
  controlP5.controller("BGLabel").setGroup(colors);
  controlP5.controller("LinesLabel").setGroup(colors);
  controlP5.controller("FillLabel").setGroup(colors);
}
public float[][][] getData(String file) {
  /* getData() loads coordinates from a file 
      and returns a coordinate array */
  
  // If file doesn't exist, return pre-existing mesh
  if (file == null)
    return mesh;
  
  // Based on the # of lines, determine size of array
  String lines[] = loadStrings(file);
  int numSteps = PApplet.parseInt(sqrt(lines.length-1)+1);
  float mesh[][][] = new float[numSteps+1][numSteps][3];
  distFromCenter = new float[numSteps][numSteps];
  
  String substrings[] = new String[3];
  float sphericalCoords[] = new float[3];

  for (int i=0; i<lines.length; i++) {
    // Convert comma-delimited ASCII data into floats
    substrings = split(lines[i],",");
    sphericalCoords = PApplet.parseFloat(substrings);
    
    // Convert spherical coordinates to Cartesian
    if (PApplet.parseInt(i/numSteps)%2 == 0 || plotLinear) {
      mesh[PApplet.parseInt(i/numSteps)][i%numSteps] = toCartesian(sphericalCoords);
      distFromCenter[PApplet.parseInt(i/numSteps)][i%numSteps] = sphericalCoords[2];
    }
    else {
      mesh[PApplet.parseInt(i/numSteps)][numSteps-i%numSteps-1] = toCartesian(sphericalCoords);
      distFromCenter[PApplet.parseInt(i/numSteps)][numSteps-i%numSteps-1] = sphericalCoords[2];
    }
  }

  // Copy the unmodified mesh over to another array
  originalMesh = new float[numSteps+1][numSteps][3];
  for (int i=0; i<numSteps; i++) {
    for (int j=0; j<numSteps-1; j++) {
      for (int k=0; k<3; k++) {
        originalMesh[i][j][k] = mesh[i][j][k];
      }
    }
  }

  for (int j=0; j<numSteps; j++) {
    // Connect the beginning and the end
    originalMesh[numSteps][j] = mesh[0][numSteps-1-j];
    mesh[numSteps][j] = mesh[0][numSteps-1-j];
  }

  for (int i=0; i<smoothingIterations; i++)
    mesh = smoothMesh(mesh);
  
  return mesh;
}

public float[] toCartesian(float input[]) {
  /* toCartesian[] converts a polar coordinate 
      to a cartesian coordinate in a float array*/
  float theta = radians(input[0]-90);
  float phi = radians(input[1]-90);
  float r = input[2];
  
  float cartCoords[] = new float[3];
  
  if (r>maximumR) {
    for (int i=0; i<3; i++) {
      cartCoords[i] = 0;
      return cartCoords;
    }
  }
    
  cartCoords[0] = r*cos(theta)*sin(phi);
  cartCoords[2] = r*sin(theta)*sin(phi);
  cartCoords[1] = r*cos(phi);
  
  return cartCoords;
}

public void drawProperties() {
  // Set drawing properties based on settings
  scale(zoomScale);
  
  // Draw axes
  if (showAxes) {
    strokeWeight(AXIS_THICKNESS);
    stroke(X_AXIS_COLOR);
    line(0,0,0,AXIS_LENGTH,0,0);
    stroke(Y_AXIS_COLOR);
    line(0,0,0,0,AXIS_LENGTH,0);
    stroke(Z_AXIS_COLOR);
    line(0,0,0,0,0,AXIS_LENGTH);
  }
  
  strokeWeight(LINE_THICKNESS);
  
  // Determine whether or not to fill
  if (wireframe)
    noFill();
  else
    fill(fillColor);
}

public boolean[][] determineFaces(float dists[][]) {
  /* Checks to see if any point is farther away from
      the center than the threshold value; if so, reject
      that point and don't plot it. */
      
  // Returns a boolean array; true means draw face, false means don't.
  boolean[][] returnBool = new boolean[dists.length-1][dists.length-1];
  
  // Initialize all entries to true
  for (int i=0; i < dists.length-1; i++) {
    for (int j=0; j < dists.length-1; j++) {
      returnBool[i][j] = true;
    }
  }
  
  for (int i=0; i < dists.length-1; i++) {
    for (int j=0; j < dists.length-1; j++) {
      if (dists[i][j] >= maximumR) {
        returnBool[i][j] = false;
        if (i != 0 && j != 0)
          returnBool[i-1][j-1] = false;
        if (i != 0)
          returnBool[i-1][j] = false;
        if (j != 0)
          returnBool[i][j-1] = false;
      }
    }
  }
  return returnBool;
}

public float distBetween(float point1[], float point2[]) {
  /* Finds the distance between two points defined by a 
      float array of length 3 (x, y, z)*/
  float sumOfSquares = 0;
  for (int i=0; i<3; i++) {
    sumOfSquares += pow(point1[i] - point2[i], 2);
  }
  return sqrt(sumOfSquares);
}

public float[][][] smoothMesh(float mesh[][][]) {
  /* smoothMesh() takes every point in the mesh and
      smooths it based on the 8 points directly surrounding it,
      with more weight on closer points. */
  float[][][] returnMesh = mesh;
  
  for (int i=0; i < mesh.length-2; i++) {
    for (int j=0; j < mesh[i].length-1; j++) {
      for (int k=0; k < 3; k++) {
        returnMesh[i][j][k] = mesh[i][j][k]/MAIN_WEIGHT;
        
        /* If i is 0, the points to the left are actually the points on
            the far right; looping around. */
        if (i == 0) {
          /* If j is 0 or the maximum value, there aren't 8 points to check, 
              so check the points that can be checked, with more weight*/
          if (j == 0) {
            returnMesh[i][j][k] += (mesh[mesh.length-2][mesh[i].length-j-2][k]+mesh[i+1][j][k]+mesh[mesh.length-2][mesh[i].length-j-1][k]+mesh[i+1][j+1][k])/DIAG_WEIGHT;
            returnMesh[i][j][k] += (mesh[i][j+1][k]+mesh[mesh[i].length-2][mesh[i].length-j-1][k]+mesh[i+1][j][k])/SIDE_WEIGHT2;
          }
          else if (j == mesh[i].length-1) {
            returnMesh[i][j][k] += (mesh[mesh.length-2][mesh[i].length-j-2][k]+mesh[i+1][j-1][k]+mesh[mesh.length-2][mesh[i].length-j][k]+mesh[i+1][j][k])/DIAG_WEIGHT;
            returnMesh[i][j][k] += (mesh[i][j-1][k]+mesh[i][j][k]+mesh[mesh[i].length-2][mesh[i].length-j-1][k]+mesh[i+1][j][k])/SIDE_WEIGHT;
          }
          /* If not at the boundary of the mesh, do this */
          else {
            returnMesh[i][j][k] += (mesh[mesh.length-2][mesh[i].length-j-2][k]+mesh[i+1][j-1][k]+mesh[mesh.length-2][mesh[i].length-j][k]+mesh[i+1][j+1][k])/DIAG_WEIGHT;
            returnMesh[i][j][k] += (mesh[i][j-1][k]+mesh[i][j+1][k]+mesh[mesh[i].length-2][mesh[i].length-j-1][k]+mesh[i+1][j][k])/SIDE_WEIGHT;
          }
        }
        else {
          /* If j is 0 or the maximum value, there aren't 8 points to check, 
              so check the points that can be checked, with more weight*/
          if (j == 0) {
            returnMesh[i][j][k] += (mesh[i-1][j][k]+mesh[i+1][j][k]+mesh[i-1][j+1][k]+mesh[i+1][j+1][k])/DIAG_WEIGHT;
            returnMesh[i][j][k] += (mesh[i][j+1][k]+mesh[i-1][j][k]+mesh[i+1][j][k])/SIDE_WEIGHT2;
          }
          else if (j == mesh[i].length-1) {
            returnMesh[i][j][k] += (mesh[i-1][j-1][k]+mesh[i+1][j-1][k]+mesh[i-1][j][k]+mesh[i+1][j][k])/DIAG_WEIGHT;
            returnMesh[i][j][k] += (mesh[i][j-1][k]+mesh[i][j][k]+mesh[i-1][j][k]+mesh[i+1][j][k])/SIDE_WEIGHT;
          }
          /* If not at the boundary of the mesh, do this */
          else {
            returnMesh[i][j][k] += (mesh[i-1][j-1][k]+mesh[i+1][j-1][k]+mesh[i-1][j+1][k]+mesh[i+1][j+1][k])/DIAG_WEIGHT;
            returnMesh[i][j][k] += (mesh[i][j-1][k]+mesh[i][j+1][k]+mesh[i-1][j][k]+mesh[i+1][j][k])/SIDE_WEIGHT;
          }
        }
      }
    }
  }
  
  for (int j=0; j<mesh[0].length-1; j++) {
    // Connect the beginning and the end
    returnMesh[mesh[0].length-1][j] = returnMesh[0][mesh[0].length-1-j];
  }
  
  return returnMesh;
}

public void smoothMultiple(int times) {
  for (int i=0; i<mesh.length; i++) {
    for (int j=0; j<mesh[i].length; j++) {
      for (int k=0; k<3; k++) {
        mesh[i][j][k] = originalMesh[i][j][k];
      }
    }
  }
  
  for (int i=0; i<times; i++) {
    mesh = smoothMesh(mesh);
  }
}

public void drawMesh(float mesh[][][]) {
  /* drawMesh() plots the points and lines/surfaces
      based on graphics settings */
  
  // Determine whether or not to draw lines, based on settings
  if (showLines || wireframe || whichShape == SHAPES[0] || whichShape == SHAPES[1])
    stroke(lineColor);
  else
    noStroke();
  
  // Begin drawing
  for (int i = 0; i < meshBool.length; i++) {
    beginShape(whichShape);
    if (useTexture && !wireframe) {  // If textures enabled
      texture(img);
    
      for (int j = 0; j < meshBool.length; j++) {
        if (meshBool[i][j]) { // determined from determineFaces()
          u = (float)i/mesh.length;
          v = (float)j/mesh[i].length;
          normal(mesh[i][j][0], mesh[i][j][1], mesh[i][j][2]);
          vertex(mesh[i][j][0], mesh[i][j][1], mesh[i][j][2],u,v);
          
          u = (float)(i+1)/mesh.length;
          normal(mesh[i+1][j][0], mesh[i+1][j][1], mesh[i+1][j][2]);
          vertex(mesh[i+1][j][0], mesh[i+1][j][1], mesh[i+1][j][2],u,v);
        } else {  
          /* If meshBool is false for this coordinate, 
              don't draw a shape, start a new one */
          endShape();
          beginShape(whichShape);
          texture(img);
        }
      }
      endShape(); // End shape for this column
    }
    else { // If textures disabled, do the following
      for (int j = 0; j < meshBool.length; j++) {
        if (meshBool[i][j]) { // determined from determineFaces()
          vertex(mesh[i][j][0], mesh[i][j][1], mesh[i][j][2]);
          vertex(mesh[i+1][j][0], mesh[i+1][j][1], mesh[i+1][j][2]);
        } else {
          /* If meshBool is false for this coordinate, 
              don't draw a shape, start a new one */
          endShape();
          beginShape(whichShape);
        }
      }
      endShape(); // End shape for this column
    }
  }
}
float currentRotation[] = {HALF_PI,PI};

// Graphics settings
boolean wireframe = false;
boolean showLines = false;
boolean useTexture = false;
boolean showAxes = true;
boolean plotLinear = false;
String imgLocation;

// Colors
int fillColor = 0xffFFFFFF;
int lineColor = 0xffFFFFFF;
int bgColor = 0xff000000;

// Zoom, depth
float zoomScale = 3.0f;
int depthTrans = 0;
int depthTransRate = 5;

// Contents of mesh array: mesh[theta][phi][x, y, or z]
float originalMesh[][][];
float mesh[][][];      // Array containing coordinates
float distFromCenter[][];
boolean meshBool[][];  // Plot this quadrilateral? True or false
int smoothingIterations = 2;
int whichShape = SHAPES[SHAPES.length-1];

// Distance threshold for plotting points
float maximumR = 255;

// For using textures
PImage img;
float u, v;

// ControlP5 menu GUI
ControlP5 controlP5;
ColorPicker color1, color2, color3;
Slider2D yzTrans;
ControlGroup menu;
ControlGroup colors;
  static public void main(String args[]) {
    PApplet.main(new String[] { "--bgcolor=#F0F0F0", "_processing__eid300_room_scan" });
  }
}
