/*  Plots a 3D mesh from a text file containing a set 
    of points defined in a spherical coordinate system; 
    coordinates are comma-delimited, and points are
    separated by a newline character. 
    
    The total number of lines in the file should be a
    square number.
*/

import processing.opengl.*;
import controlP5.*;

void setup() {
  size(800, 600, OPENGL);
  
  // Load texture and data from default files
  img = loadImage("default.png");
  mesh = getData("default.txt");
  meshBool = determineFaces(distFromCenter);

  /* NORMALIZED allows texture coordinates to
       be defined relative to 1 in drawMesh()*/
  textureMode(NORMALIZED);
  
  enableScrollWheel();
  menuSetup();
}

void draw() {
  // Update colors based on color selections
  bgColor = color1.getColorValue();
  lineColor = color2.getColorValue();
  fillColor = color3.getColorValue();
  
  background(bgColor);
  
  // Plot points
  pushMatrix();
  lights();
  translate(width/2-yzTrans.arrayValue()[0],
            height/2-yzTrans.arrayValue()[1],depthTrans);
  rotateCamera();
  drawProperties();
  drawMesh(mesh);
  popMatrix();
  
  // Draw menu
  hint(DISABLE_DEPTH_TEST);
  strokeWeight(1);
  controlP5.draw();
  hint(ENABLE_DEPTH_TEST);
}
