public void enableScrollWheel() {
  /* Allows the user to zoom in/out using scroll wheel 
      http://wiki.processing.org/index.php/Wheel_mouse
      Code by Rick Companje */
  addMouseWheelListener(
    new java.awt.event.MouseWheelListener() { 
      public void mouseWheelMoved(java.awt.event.MouseWheelEvent evt) { 
        zoom(evt.getWheelRotation());
      }
    }
  ); 
}

public void keyPressed() {
  // Allows zooming and changing depth via the arrow keys
  if (key == CODED) {
    switch(keyCode) {
      case UP:
        zoom(1);
        break;
      case DOWN:
        zoom(-1);
        break;
      case RIGHT:
        if (depthTrans < DEPTH_TRANS_MAX)
          depthTrans += depthTransRate;
          controlP5.controller("Depth").setValue(depthTrans);
        break;
      case LEFT:
        if (depthTrans > 0)
          depthTrans -= depthTransRate;      
          controlP5.controller("Depth").setValue(depthTrans);
        break;
    }
  }
}

public void rotateCamera() {  
  // Rotate the camera on a mouse click+drag
  if (mousePressed) {
    /* Check to see if menus are open 
        Don't rotate if the mouse is above a menu */
    if (!( menu.isOpen() && mouseX < MENU_XBOUND ) &&
        !( colors.isOpen() && mouseX > width-MENU_XBOUND 
            && mouseY < COLORS_MENU_HEIGHT ) ) {
      currentRotation[0] += RCONST*(pmouseY-mouseY);
      currentRotation[1] += RCONST*(pmouseX-mouseX);
  
      /* Rotation about the x-axis is 
          limited between 0 and PI */
      if (currentRotation[0] > PI)
        currentRotation[0] = PI;
      if (currentRotation[0] < 0)
        currentRotation[0] = 0;

      currentRotation[1] %= TWO_PI;
    }
  }
  rotateX(currentRotation[0]);
  rotateZ(currentRotation[1]);
}

public void zoom(int delta) {
  /* Positive delta means zoom in 
     Negative delta means zoom out */
     
  zoomScale += delta*ZOOM_RATE;
  if (zoomScale < ZOOM_RANGE[0])
    zoomScale = ZOOM_RANGE[0];
  if (zoomScale > ZOOM_RANGE[1])
    zoomScale = ZOOM_RANGE[1];
  controlP5.controller("Zoom").setValue(zoomScale);
}
