public void controlEvent(ControlEvent theEvent) {
  // Event handling for menu clicks
  switch(theEvent.controller().id()) { // Check ID
    case(1): // maximumR adjust
      maximumR = theEvent.controller().value();
      meshBool = determineFaces(distFromCenter);
      break;
    case(2): // Depth translation
      depthTrans = (int) theEvent.controller().value();
      break;
    case(3): // Zoom in/out
      zoomScale = theEvent.controller().value();
      break;
    case(4): // Smoothing
      smoothingIterations = (int) theEvent.controller().value();
      smoothMultiple(smoothingIterations);
      break;
    case(5): // Shape
      whichShape = SHAPES[(int) theEvent.controller().value()-1];
      break;
    case(6): // Load texture
      imgLocation = selectInput();
      if (imgLocation != null)
        img = loadImage(selectInput());
      break;
    case(7): // getData()
      mesh = getData(selectInput());
      meshBool = determineFaces(distFromCenter);
      break;
  }
}

public void menuSetup() {
  // menuSetup() configures the main menu and colors menu
  controlP5 = new ControlP5(this);
  controlP5.setAutoDraw(false); 
  
  menu = controlP5.addGroup("Menu",0,10);
  menu.setBackgroundHeight(height);
  menu.setBackgroundColor(MENU_BG_COLOR);
  menu.setWidth(MENU_XBOUND);
  
  // Toggle settings with clickable buttons
  controlP5.addToggle("wireframe",wireframe,25,10,BUTTON_WIDTH,BUTTON_WIDTH).setGroup(menu);
  controlP5.addToggle("useTexture",useTexture,25,60,BUTTON_WIDTH,BUTTON_WIDTH).setGroup(menu);
  controlP5.addToggle("showAxes",showAxes,25,110,BUTTON_WIDTH,BUTTON_WIDTH).setGroup(menu);
  controlP5.addToggle("showLines",showLines,25,160,BUTTON_WIDTH,BUTTON_WIDTH).setGroup(menu);
  
  // Rename button labels
  controlP5.controller("useTexture").setLabel("Use Texture");
  controlP5.controller("showAxes").setLabel("Show Axes");
  controlP5.controller("showLines").setLabel("Show Lines");
  
  // Add sliders for camera and rendering settings
  controlP5.addSlider("Threshold",0,300,300,0,210,MENU_XBOUND,BUTTON_WIDTH).setId(1);
  controlP5.controller("Threshold").setGroup(menu);
  controlP5.addSlider("Depth",0,DEPTH_TRANS_MAX,0,0,240,MENU_XBOUND,BUTTON_WIDTH).setId(2);
  controlP5.controller("Depth").setGroup(menu);
  controlP5.addSlider("Zoom",ZOOM_RANGE[0],ZOOM_RANGE[1],0,0,270,MENU_XBOUND,BUTTON_WIDTH).setId(3);
  controlP5.controller("Zoom").setGroup(menu);
 
  controlP5.addSlider("Smoothing",0,MAX_SMOOTHING,2,0,300,MENU_XBOUND,BUTTON_WIDTH/2).setId(4);
  Slider smoothSlider = (Slider) controlP5.controller("Smoothing");
  smoothSlider.setGroup(menu);
  smoothSlider.setSliderMode(Slider.FLEXIBLE);
  smoothSlider.setNumberOfTickMarks(MAX_SMOOTHING+1);

  controlP5.addSlider("Shape",1,SHAPES.length,SHAPES.length,0,330,MENU_XBOUND,BUTTON_WIDTH/2).setId(5);
  Slider shapeSlider = (Slider) controlP5.controller("Shape");
  shapeSlider.setGroup(menu);
  shapeSlider.setSliderMode(Slider.FLEXIBLE);
  shapeSlider.setNumberOfTickMarks(SHAPES.length);
  
  // Buttons to load texture/data from file
  controlP5.addButton("Load texture",0,0,380,MENU_XBOUND,20).setId(6);
  controlP5.addButton("Load data",0,0,400,MENU_XBOUND,20).setId(7);
  controlP5.controller("Load texture").setGroup(menu);
  controlP5.controller("Load data").setGroup(menu);
  
  /* addSlider2D(name, xMin, xMax, yMin, yMax, defaultX, defaultY, 
                 xPos, yPos, width, height); */
  yzTrans = controlP5.addSlider2D("YZ-Translate",-width/4,width/4,
                  -height/4,height/4,width/4,height/4,0,height-125,MENU_XBOUND,MENU_XBOUND);
  yzTrans.setGroup(menu);
  
  // Colors menu group
  colors = controlP5.addGroup("Colors",width-100,10);
  colors.setBackgroundHeight(COLORS_MENU_HEIGHT);
  colors.setBackgroundColor(MENU_BG_COLOR);
  colors.setWidth(MENU_XBOUND);
  
  // Add color pickers to colors menu
  color1 = controlP5.addColorPicker("BG",0,175,MENU_XBOUND,10);
  color2 = controlP5.addColorPicker("Lines",0,15,MENU_XBOUND,10);
  color3 = controlP5.addColorPicker("Fill",0,95,MENU_XBOUND,10);
  color1.setColorValue(bgColor);
  color2.setColorValue(lineColor);
  color3.setColorValue(fillColor);
  color1.setGroup(colors);
  color2.setGroup(colors);
  color3.setGroup(colors);
  
  // Labels for color pickers
  controlP5.addTextlabel("BGLabel","Background",5,165);
  controlP5.addTextlabel("LinesLabel","Lines",5,5);
  controlP5.addTextlabel("FillLabel","Fill",5,85);
  controlP5.controller("BGLabel").setGroup(colors);
  controlP5.controller("LinesLabel").setGroup(colors);
  controlP5.controller("FillLabel").setGroup(colors);
}
